@@ -11,6 +11,14 @@ typedef shl_t minthandle_t;
 #include <windows.h>
 typedef HMODULE minthandle_t;
 #undef WIN32_LEAN_AND_MEAN
+#elif defined(OS_darwin)
+#define DL_LOADONCEONLY
+#define DLSYMUN
+#undef environ
+#undef bool
+
+#import <mach-o/dyld.h>
+typedef void *minthandle_t; 
 #else 
 #include <dlfcn.h>
 typedef void *minthandle_t; 
@@ -34,11 +42,12 @@ typedef struct {
   void *store;
 } UTIL_args;
 
+#if 0
 int getobj(void *hv_t,char *key,char *type,void **store);
 int getint(void *hv_t,char *key,int *store);
 int getdouble(void *hv_t,char *key,double *store);
 int getvoid(void *hv_t,char *key,void **store);
-
+#endif
 
 void *DSO_open(char* file,char** evalstring);
 int DSO_close(void *);
