@@ -1,10 +1,8 @@
 package Imager::Font::Win32;
 use strict;
-use vars qw(@ISA $VERSION);
+use vars qw(@ISA);
 @ISA = qw(Imager::Font);
 
-$VERSION = sprintf "%d.%03d", q$Revision: 1.4 $=~/\d+/g;
-
 # called by Imager::Font::new()
 # since Win32's HFONTs include the size information this
 # is just a stub
@@ -17,7 +15,7 @@ sub new {
 sub _bounding_box {
   my ($self, %opts) = @_;
   
-  my @bbox = Imager::i_wf_bbox($self->{face}, $opts{size}, $opts{string});
+  my @bbox = i_wf_bbox($self->{face}, $opts{size}, $opts{string});
 }
 
 sub _draw {
